//=======================================================================
//			Copyright (C) Shambler Team 2004
//			    m40a1.cpp - weapon_ m40a1
//			    
//=======================================================================

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "weapons.h"
#include "monsters.h"
#include "player.h"
#include "defaults.h"

enum shiperrifle_e {
	M40A1_IDLE = 0,
	M40A1_IDLE2,
	M40A1_DRAW,
	M40A1_HOLSTER,
	M40A1_FIRE,
	M40A1_FIRE_EMPTY,
	M40A1_RELOAD1,
	M40A1_RELOAD2,
	M40A1_RELOAD3,
};

class CSniper : public CBasePlayerWeapon
{
public:
	void Spawn( void );
	void Precache( void );
	int GetItemInfo(ItemInfo *p);
	void PrimaryAttack( void );
	void SecondaryAttack( void ) { m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 1E6; };//just stub
	BOOL Deploy( void );
	void Holster( );
	void Reload( void );
	void WeaponIdle( void );
	void ZoomUpdate( void );
	void ZoomReset( void );
	BOOL b_setup;
	BOOL ShouldWeaponIdle( void ) { return TRUE; };
private:
	unsigned short m_usFireSniper;
};

LINK_ENTITY_TO_CLASS( weapon_m40a1, CSniper );

void CSniper::Spawn( )
{
	Precache( );
	UTIL_SetModel(ENT(pev), "materials/weapons/m40a1/wp_m40a1.mdl");

	if(IsMultiplayer()) m_iDefaultAmmo = M40A1_RANDOM_GIVE;
	else m_iDefaultAmmo = M40A1_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}

int CSniper::GetItemInfo(ItemInfo *p)
{
 	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "762";
	p->iMaxAmmo1 = M40A1_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = M40A1_MAX_CLIP;
	p->iFlags = 0;
	p->iSlot = SLOT_3;
	p->iPosition = POS_1;
	p->iId = m_iId = WEAPON_M40A1;
	p->iWeight = M40A1_WEIGHT;

	return 1;
}

void CSniper::Precache( void )
{
	vModel = UTIL_PrecacheWeaponModel("materials/weapons/m40a1/v_m40a1.mdl");
	wModel = UTIL_PrecacheWeaponModel("materials/weapons/m40a1/wp_m40a1.mdl");
	UTIL_PrecacheSound("weapons/m40a1/zoom.wav");
          UTIL_PrecacheModel("materials/gibs/shell762.mdl");
	m_usFireSniper = UTIL_PrecacheEvent( "materials/events/sniper.sc" );
}

BOOL CSniper::Deploy( )
{
	return DefaultDeploy( M40A1_DRAW, "shotgun" );
}


void CSniper::Holster( )
{
	ZoomReset();
	m_fInReload = FALSE;// cancel any reload in progress.
	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 1.0;
	SendWeaponAnim( M40A1_HOLSTER );
}

void CSniper::PrimaryAttack()
{
	if ( m_iClip && m_pPlayer->pev->waterlevel != 3)//don't fire underwater
	{
		m_pPlayer->m_iWeaponVolume = LOUD_GUN_VOLUME;
		m_pPlayer->m_iWeaponFlash = NO_GUN_FLASH;

		m_iClip--;

		// player "shoot" animation
		m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

		UTIL_MakeVectors( m_pPlayer->pev->v_angle + m_pPlayer->pev->punchangle );

		Vector vecSrc	 = m_pPlayer->GetGunPosition( );
		Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_10DEGREES );

		Vector vecDir;
		vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_1DEGREES, 8192, BULLET_PLAYER_357, 0, 0, m_pPlayer->pev, m_pPlayer->random_seed );

		PLAYBACK_EVENT_FULL( 0, m_pPlayer->edict(), m_usFireSniper, 0.0, (float *)&g_vecZero, (float *)&g_vecZero, vecDir.x, vecDir.y, pev->body, 0, ( m_iClip == 0 ) ? 1 : 0, 0 );
                    ZoomReset();
		m_flNextPrimaryAttack = gpGlobals->time + 2.0;
		m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + RANDOM_FLOAT ( 10, 15 );
	}
	else
	{
		PlayEmptySound();
		m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.7;
	}
}

void CSniper::Reload( void )
{
	if ( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0) return;
	if ( m_iChargeLevel ) ZoomReset();
	if(m_iClip) DefaultReload( M40A1_MAX_CLIP, M40A1_RELOAD3, 2.5 );
          else 	  DefaultReload( M40A1_MAX_CLIP, M40A1_RELOAD1, 2.5 );
          m_iOverloadLevel = 1;
	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + RANDOM_FLOAT ( 10, 15 );
}

void CSniper::ZoomUpdate( void )
{
	if (m_pPlayer->pev->button & IN_ATTACK2)
	{
		if(m_iChargeLevel == 0)
		{
			if (m_flShockTime > UTIL_WeaponTimeBase()) return;
			m_iChargeLevel = 1;
			EMIT_SOUND(ENT(m_pPlayer->pev), CHAN_ITEM, "weapons/m40a1/zoom.wav", 1, ATTN_NORM);
			m_flTimeUpdate = UTIL_WeaponTimeBase() + 0.8;
		}
		if(m_iChargeLevel == 1)
		{
			m_pPlayer->m_iFOV = 50;
			m_pPlayer->pev->viewmodel = NULL;
			m_iChargeLevel = 2;//ready to zooming, wait for 0.8 secs
		}

		
		if (m_iChargeLevel == 2 && m_pPlayer->m_iFOV > MAX_ZOOM)
		{
			if (m_flTimeUpdate < UTIL_WeaponTimeBase())
			{
				m_pPlayer->m_iFOV--;
				m_flTimeUpdate = UTIL_WeaponTimeBase() + 0.02;
			}
		}
		if(m_iChargeLevel == 3) ZoomReset();
	}
	else if(m_iChargeLevel > 1) m_iChargeLevel = 3;

	MESSAGE_BEGIN( MSG_ONE, gmsgZoomHUD, NULL, m_pPlayer->pev );
		WRITE_BYTE( m_iChargeLevel );
	MESSAGE_END();
}

void CSniper::ZoomReset( void )
{
	//return viewmodel
	m_pPlayer->pev->viewmodel = vModel;
	m_flShockTime = UTIL_WeaponTimeBase() + 0.5;
	m_pPlayer->m_iFOV = 90;
	m_iChargeLevel = 0;//clear zoom
	MESSAGE_BEGIN( MSG_ONE, gmsgZoomHUD, NULL, m_pPlayer->pev );
		WRITE_BYTE( m_iChargeLevel );
	MESSAGE_END();
	m_pPlayer->UpdateClientData();//update client data manually
}

void CSniper::WeaponIdle( void )
{
	ZoomUpdate();

	m_pPlayer->GetAutoaimVector( AUTOAIM_2DEGREES );
	if (m_iOverloadLevel)
	{
		SendWeaponAnim( M40A1_RELOAD2 );
		m_iOverloadLevel = 0; //reloaded ok
	}
	if ( m_flTimeWeaponIdle > UTIL_WeaponTimeBase() )return;

	if(m_iClip)
	{
		float flRand = RANDOM_FLOAT(0, 1);
		if (flRand <= 0.3)
		{
			SendWeaponAnim(M40A1_IDLE);
			m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 101.0/23.0;
		}
		else	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + RANDOM_LONG(10, 30);
	}
	else
	{         //enable animation instantly
		SendWeaponAnim(M40A1_IDLE2);
          	m_flTimeWeaponIdle = UTIL_WeaponTimeBase() + 101.0/23.0;
	}
}
